"""
    created 30.05.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import math


def human_seconds(duration):
    minutes = math.floor(duration/60)
    seconds = math.floor(duration - (minutes*60))
    return "%i:%02i" % (minutes, seconds)



import io
import math
from pathlib import Path

import svgwrite

# https://github.com/jedie/django-for-runners
# Django CMS Tools
from for_runners.gpx_tools.garmin2gpxpy import garmin2gpxpy


def gpx2svg(gpxpy_instance):
    lat_list = []
    lon_list = []

    for track in gpxpy_instance.tracks:
        for segment in track.segments:
            for point in segment.points:
                # print('Point at ({0},{1}) -> {2}'.format(point.latitude, point.longitude, point.elevation))
                lat_list.append(point.latitude)
                lon_list.append(point.longitude)

    # print(lat)
    # print(lon)

    lon_min = min(lon_list)
    lat_min = min(lat_list)
    lon_max = max(lon_list)
    lat_max = max(lat_list)

    print(
        "lon",
        lon_min,
        lon_max,
        "lat",
        lat_min,
        lat_max,
    )

    lat_area = lat_max - lat_min
    lon_area = lon_max - lon_min
    if lon_area > lat_area:
        aspect = lat_area / lon_area
    else:
        aspect = lon_area / lat_area
    print("areas:", lat_area, lon_area, "aspect:", aspect)

    ############################################################################################

    border = 5

    # the minimum distance between two points
    min_distance = 0.2

    total_size_x = 100
    total_size_y = 100

    print("total_size:", total_size_x, total_size_y)

    drawing = svgwrite.Drawing(size=(total_size_x, total_size_y), profile='tiny')
    drawing.add(drawing.rect(insert=(0, 0), size=(total_size_x, total_size_y), fill='#000000'))
    drawing.add(drawing.rect(insert=(1, 1), size=(total_size_x - 2, total_size_y - 2), fill='#ffffff'))
    lines = drawing.add(drawing.g(stroke_width=1, stroke='blue', fill='none'))

    lines_size_x = total_size_x - (border * 2)
    lines_size_y = total_size_y - (border * 2)
    print("lines_size:", lines_size_x, lines_size_y)

    scale_x = lines_size_x / lon_area * aspect
    scale_y = lines_size_y / lat_area

    print("scale:", scale_x, scale_y)

    max_x = lon_area * scale_x
    max_y = lat_area * scale_y
    print("max:", max_x, max_y)

    offset_x = border + ((lines_size_x - max_x) / 2)
    offset_y = border + ((lines_size_y - max_y) / 2)
    print("offset:", offset_x, offset_y)

    # x_list = []
    # y_list = []

    old_x = None
    old_y = None
    for lon, lat in zip(lon_list, lat_list):
        x = ((lon - lon_min) * scale_x) + offset_x
        y = ((lat - lat_min) * scale_y) + offset_y

        y = y * -1 + total_size_y  # mirror the x-axis

        # x_list.append(x)
        # y_list.append(y)

        if old_x is not None:
            if abs(old_x - x) < min_distance:
                continue
            if abs(old_y - y) < min_distance:
                continue
            lines.add(drawing.line(start=(old_x, old_y), end=(x, y)))

        old_x = x
        old_y = y

    # print(min(x_list), max(x_list), min(y_list), max(y_list))

    return drawing


def gpx2svg_file(gpxpy_instance, svg_filename, pretty=False):
    drawing = gpx2svg(gpxpy_instance)
    drawing.saveas(svg_filename, pretty=pretty)


def gpx2svg_string(gpxpy_instance, pretty=False):
    drawing = gpx2svg(gpxpy_instance)
    fileobj = io.StringIO()
    drawing.write(fileobj, pretty=False)
    return fileobj.getvalue()





# coding: utf-8




from django.conf.urls import include, url, static
from django.conf.urls.i18n import i18n_patterns
from django.contrib import admin

from for_runners_test_project import settings


admin.autodiscover()


urlpatterns = i18n_patterns(
    url(r'^admin/', include(admin.site.urls)),
    url(r'^', include('cms.urls')),
)

if settings.DEBUG:
    urlpatterns += static.static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)
    urlpatterns += static.static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

    import debug_toolbar
    urlpatterns = [
        url(r'^__debug__/', include(debug_toolbar.urls)),
    ] + urlpatterns



"""
    created 30.05.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
import io
import logging

from django.conf import settings
from django.core.files import File
from django.db import models
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from filer.fields.file import FilerFileField
from filer.utils.loader import load_model

# https://github.com/jedie/django-tools
from django_tools.models import UpdateInfoBaseModel, UpdateTimeBaseModel

# https://github.com/jedie/django-for-runners
from for_runners.geo import reverse_geo
from for_runners.gpx import get_identifier, parse_gpx
from for_runners.gpx_tools.humanize import human_seconds
from for_runners.managers import GpxModelManager
from for_runners.svg import gpx2svg_string

log = logging.getLogger(__name__)


class DisciplineModel(models.Model):
    name = models.CharField(max_length=255, help_text=_("Sport discipline"))

    def __str__(self):
        return self.name


class EventModel(UpdateInfoBaseModel):
    """
    inherit from UpdateInfoBaseModel:
        * createtime
        * lastupdatetime
        * createby
        * lastupdateby
    """
    no = models.IntegerField(
        help_text=_("Sequential number of the event"),
        null=True, blank=True,
    )
    name = models.CharField(max_length=255, help_text=_("Name of the event"))
    start_time = models.DateTimeField(help_text=_("Start date/time of the run"),
        null=True, blank=True,
    )
    discipline = models.ForeignKey(DisciplineModel)

    def __str__(self):
        return "%i. %s %s" % (self.no, self.name, self.start_time)


class GpxModel(UpdateTimeBaseModel):
    """
    inherit from UpdateTimeBaseModel:
        * createtime
        * lastupdatetime
    """
    event = models.ForeignKey(
        EventModel,
        null=True,
        blank=True,
    )

    gpx = models.TextField(help_text="The raw gpx file content",)
    track_svg = FilerFileField(verbose_name=_("Track SVG"), related_name="+",
        null=True,
        blank=True,
    )

    start_time = models.DateTimeField(editable=False,
        help_text=_("Start time of the first segment in track"),
    )
    start_latitude = models.FloatField(editable=False,
        help_text=_("Latitude of the first recorded point from the *.gpx file"),
    )
    start_longitude = models.FloatField(editable=False,
        help_text=_("Longitude of the first recorded point from the *.gpx file"),
    )
    short_start_address = models.CharField(
        max_length=255,
        null=True,
        blank=True,
        help_text="The short address of the start point",
    )
    full_start_address = models.CharField(
        max_length=255,
        null=True,
        blank=True,
        help_text="The full address of the start point",
    )

    finish_time = models.DateTimeField(editable=False,
        help_text=_("End time of the last segment in track"),
    )
    finish_latitude = models.FloatField(editable=False,
        help_text=_("Latitude of the finish point"),
    )
    finish_longitude = models.FloatField(editable=False,
        help_text=_("Longitude of the finish point"),
    )
    short_finish_address = models.CharField(
        max_length=255,
        null=True,
        blank=True,
        help_text="The short address of the finish point",
    )
    full_finish_address = models.CharField(
        max_length=255,
        null=True,
        blank=True,
        help_text="The full address of the finish point",
    )

    tracked_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        editable=False,
        related_name="%(class)s_createby",
        null=True,
        blank=True,
        help_text="The user that tracked this gpx entry",
        on_delete=models.SET_NULL
    )
    lastupdateby = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        editable=False,
        related_name="%(class)s_lastupdateby",
        null=True,
        blank=True,
        help_text="User as last edit this entry",
        on_delete=models.SET_NULL
    )



    points_no = models.PositiveIntegerField(
        help_text=_("Number of points in GPX"),
        null=True, blank=True,
    )

    length = models.PositiveIntegerField(
        help_text=_("Length in meters (calculated 3-dimensional used latitude, longitude, and elevation)"),
        null=True, blank=True,
    )
    duration = models.PositiveIntegerField(
        help_text=_("Duration in seconds"),
        null=True, blank=True,
    )
    pace = models.DecimalField(
        help_text=_("Min/km (number of minutes it takes to cover a kilometer)"),
        max_digits=4, decimal_places=2, # store numbers up to 99 with a resolution of 2 decimal places
        null=True, blank=True,
    )

    uphill = models.IntegerField(
        help_text=_("Uphill elevation climbs in meters"),
        null=True, blank=True,
    )
    downhill = models.IntegerField(
        help_text=_("Downhill elevation descent in meters"),
        null=True, blank=True,
    )
    min_elevation = models.IntegerField(
        help_text=_("Minimum elevation in meters"),
        null=True, blank=True,
    )
    max_elevation = models.IntegerField(
        help_text=_("Maximum elevation in meters"),
        null=True, blank=True,
    )

    map_image = models.ImageField(
        null=True,
        blank=True,
    )

    objects = GpxModelManager()

    def save(self, *args, **kwargs):
        if self.gpx:
            self.calculate_values()

        super().save(*args, **kwargs)

    def svg_tag(self):
        if self.track_svg:
            return '<img src="{}" alt="gpx track" height="70px" width="70px" />'.format(self.track_svg.url)
        return ""

    svg_tag.short_description = _("SVG")
    svg_tag.allow_tags = True

    def svg_tag_big(self):
        if self.track_svg:
            return '<img src="{}" alt="gpx track" height="200px" width="200px" />'.format(self.track_svg.url)
        return ""

    svg_tag_big.short_description = _("SVG")
    svg_tag_big.allow_tags = True

    def image_tag(self):
        if self.map_image:
            return '<img src="%s" />' % self.map_image.url
        return ""

    image_tag.short_description = _("Map Image")
    image_tag.allow_tags = True

    def human_length(self):
        if self.length:
            kilometers = self.length / 1000
            return "%.2f km" % kilometers
    human_length.short_description = _("Length")

    def human_duration(self):
        if self.duration:
            return human_seconds(self.duration)
    human_duration.short_description = _("Duration")

    def human_pace(self):
        if self.pace:
            return "%s min/km" % human_seconds(self.pace * 60)
    human_pace.short_description = _("Pace")

    def _coordinate2link(self, lat, lon):
        return (
            '<a'
            ' href="https://nominatim.openstreetmap.org/reverse?lat={lat}&lon={lon}&format=json&addressdetails=1"'
            ' target="_blank"'
            '>'
            '{lat},{lon}'
            '</a>'
        ).format(
            lat=lat, lon=lon
        )

    def start_coordinate_html(self):
        """
        https://nominatim.openstreetmap.org/reverse?lat=51.20638179592788&lon=6.803598012775183&format=json&addressdetails=1
        """
        return self._coordinate2link(
            lat=self.start_latitude,
            lon=self.start_longitude,
        )

    start_coordinate_html.short_description = _("Start coordinates")
    start_coordinate_html.allow_tags = True

    def get_gpxpy_instance(self):
        gpxpy_instance = parse_gpx(content=self.gpx)
        return gpxpy_instance

    def calculate_values(self):
        gpxpy_instance = self.get_gpxpy_instance()
        self.points_no = gpxpy_instance.get_points_no()
        self.length = gpxpy_instance.length_3d()
        self.duration = gpxpy_instance.get_duration()
        self.pace = (self.duration / 60) / (self.length / 1000)

        uphill_downhill = gpxpy_instance.get_uphill_downhill()
        self.uphill = uphill_downhill.uphill
        self.downhill = uphill_downhill.downhill

        elevation_extremes = gpxpy_instance.get_elevation_extremes()
        self.min_elevation = elevation_extremes.minimum
        self.max_elevation = elevation_extremes.maximum

        identifier = get_identifier(gpxpy_instance)
        self.start_time = identifier.start_time
        self.finish_time = identifier.finish_time
        self.start_latitude = identifier.start_lat
        self.start_longitude = identifier.start_lon
        self.finish_latitude = identifier.finish_lat
        self.finish_longitude = identifier.finish_lon

        try:
            start_address = reverse_geo(self.start_latitude, self.start_longitude)
        except Exception as err:
            # e.g.: geopy.exc.GeocoderTimedOut: Service timed out
            log.error("Can't reverse geo: %s" % err)
        else:
            self.short_start_address = start_address.short
            self.full_start_address = start_address.full

        try:
            finish_address = reverse_geo(self.finish_latitude, self.finish_longitude)
        except Exception as err:
            # e.g.: geopy.exc.GeocoderTimedOut: Service timed out
            log.error("Can't reverse geo: %s" % err)
        else:
            self.short_finish_address = finish_address.short
            self.full_finish_address = finish_address.full

        # if not self.track_svg:
        log.debug("Create SVG from GPX...")
        svg_string = gpx2svg_string(gpxpy_instance)

        # import filer.models.imagemodels.Image
        Image = load_model(settings.FILER_IMAGE_MODEL)

        temp = io.BytesIO(bytes(svg_string, "utf-8"))
        django_file_obj = File(temp, name="gpx.svg")
        filer_image = Image.objects.create(
            owner=self.tracked_by, original_filename="gpx.svg", file=django_file_obj, folder=None
        )
        filer_image.save()

        # self.track_svg.save("gpx2svg", svg_string)
        self.track_svg = filer_image  #save("gpx2svg", svg_string)

    def __str__(self):
        parts = [self.start_time, self.event, self.short_start_address]
        result = " ".join([str(part) for part in parts if part])
        if result:
            return result
        return "GPX Track ID:%s" % self.pk

    class Meta:
        verbose_name = _('GPX Track')
        verbose_name_plural = _('GPX Tracks')
        unique_together = ((
            "start_time", "start_latitude", "start_longitude", "finish_time", "finish_latitude", "finish_longitude"
        ),)
        ordering = ('-start_time', '-pk')



"""
    created 31.05.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
import collections
import io
from pathlib import Path

import gpxpy

# https://github.com/jedie/django-for-runners
from for_runners.gpx_tools.garmin2gpxpy import garmin2gpxpy

Identifier = collections.namedtuple('Identifier', ('start_time, start_lat, start_lon, finish_time, finish_lat, finish_lon'))


def get_identifier(gpxpy_instance):
    """
    :return: Identifier named tuple
    """
    time_bounds = gpxpy_instance.get_time_bounds()

    first_track = gpxpy_instance.tracks[0]
    first_segment = first_track.segments[0]
    first_point = first_segment.points[0]

    last_track = gpxpy_instance.tracks[-1]
    last_segment = last_track.segments[-1]
    last_point = last_segment.points[-1]

    return Identifier(
        time_bounds.start_time, first_point.latitude, first_point.longitude, time_bounds.end_time, last_point.latitude,
        last_point.longitude
    )


def parse_gpx(content):
    if 'creator="Garmin Connect"' in content:
        # work-a-round until https://github.com/tkrajina/gpxpy/issues/115#issuecomment-392798245 fixed
        return garmin2gpxpy(content)

    temp = io.BytesIO(content)
    return gpxpy.parse(temp)



"""
    created 30.05.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
import asyncio

import geotiler  # https://wrobell.dcmod.org/geotiler/
import matplotlib.pyplot as plt  # https://pypi.org/project/matplotlib/


def generate_map(gpxpy_instance):
    lat = []
    lon = []
    for track in gpxpy_instance.tracks:
        for segment in track.segments:
            for point in segment.points:
                # print('Point at ({0},{1}) -> {2}'.format(point.latitude, point.longitude, point.elevation))
                lat.append(point.latitude)
                lon.append(point.longitude)

    # print(lat)
    # print(lon)

    lon_min = min(lon)
    lat_min = min(lat)
    lon_max = max(lon)
    lat_max = max(lat)

    print(lon_min, lon_max, lat_min, lat_max)

    width = lat_max - lat_min
    height = lon_max - lon_min
    print("width", width, "height", height)

    aspect = width / height  # 0.8114047529717512 -> 1280 1039
    print("aspect", aspect)

    size_x = 1280
    size_y = int(round(size_x * aspect, 0))
    print(size_x, size_y)

    map = geotiler.Map(extent=(lon_min, lat_min, lon_max, lat_max), size=(size_x, size_y))
    lon_min2, lat_min2, lon_max2, lat_max2 = map.extent

    print("Render map...")
    asyncio.set_event_loop(asyncio.new_event_loop())  # Fix: There is no current event loop in thread 'Thread-1'.
    image = geotiler.render_map(map)
    print("OK")

    ax = plt.subplot(111)
    ax.imshow(image, extent=(lon_min2, lon_max2, lat_min2, lat_max2), aspect='auto', alpha=0.4)

    plt.plot(lon, lat, color="#000000", lw=0.5, alpha=0.9)

    time_bounds = gpxpy_instance.get_time_bounds()
    plt.title(time_bounds.start_time)

    plt.xlabel('latitude')
    plt.ylabel('longitude')

    return image, plt


# gpx = garmin2gpxpy(filepath=FILEPATH)
#
# print("length: %.2fm" % gpx.length_3d())
# print("duration: %.2fs" % gpx.get_duration())
# print("uphill_downhill:", gpx.get_uphill_downhill())
#
# time_bounds = gpx.get_time_bounds()
# print(time_bounds)
#
# image, plt = generate_map(gpx)
#
# image.save('test_map.png')
# plt.savefig('test.png', bbox_inches='tight')
# # plt.show()



# coding: utf-8



import os
from unittest import TestCase

import pytest

from django.core.management import call_command

from django_tools.unittest_utils.django_command import DjangoCommandMixin

import for_runners_test_project

MANAGE_DIR = os.path.abspath(os.path.dirname(for_runners_test_project.__file__))


class CheckTestEnvironment(DjangoCommandMixin, TestCase):

    def call_manage_py(self, *args, **kwargs):
        return super(CheckTestEnvironment, self).call_manage_py(
            args, manage_dir=MANAGE_DIR, **kwargs
        )

    def test_help(self):
        output = self.call_manage_py("--help")
        print(output)
        self.assertNotIn("ERROR", output)
        self.assertIn("[django]", output)
        self.assertIn("[cms]", output)
        self.assertIn("Type 'manage.py help <subcommand>' for help on a specific subcommand.", output)

    def test_django_check(self):
        output = self.call_manage_py("check")
        self.assertIn("System check identified no issues (0 silenced).", output)

    @pytest.mark.django_db
    def test_cms_check(self):
        """
        We can't call "cms check" via ./manage.py because the database
        doesn't exist, so it will raise errors.
        """
        call_command("cms", "check")

        # output = self.call_manage_py("cms", "check")
        # print(output)
        # self.assertNotIn("error", output)
        # self.assertIn("Installation okay", output)




"""
    created 30.05.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
import io
import logging

from django import forms
from django.conf.urls import url
from django.contrib import admin, messages
from django.db import models
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from django.views import generic

# https://github.com/jedie/django-for-runners
from for_runners.forms import UploadGpxFileForm
from for_runners.gpx_tools.garmin2gpxpy import garmin2gpxpy
from for_runners.gpx_tools.gpxpy2map import generate_map
from for_runners.models import DisciplineModel, EventModel, GpxModel

log = logging.getLogger(__name__)


@admin.register(DisciplineModel)
class DisciplineModelAdmin(admin.ModelAdmin):
    pass


@admin.register(EventModel)
class EventModelAdmin(admin.ModelAdmin):
    pass


class UploadGpxFileView(generic.FormView):
    template_name = "for_runners/upload_gpx_file.html"
    form_class = UploadGpxFileForm
    success_url = "../"  # FIXME

    def post(self, request, *args, **kwargs):
        form_class = self.get_form_class()
        form = self.get_form(form_class)
        files = request.FILES.getlist("gpx_files")
        if form.is_valid():
            log.debug("files: %r", files)
            for f in files:
                if f.content_type != "application/gpx+xml":
                    messages.error(request, "Ignore %r file %r!" % (f.content_type, f.name))
                    continue
                messages.info(request, "Process %s..." % f.name)

                content = f.file.read()
                log.debug("raw content......: %s", repr(content)[:100])
                content = content.decode("utf-8")
                log.debug("decoded content..: %s", repr(content)[:100])

                gpx = GpxModel.objects.create(gpx=content)
                gpx.calculate_values()
                messages.success(request, "Created: %s" % gpx)

            return self.form_valid(form)
        else:
            return self.form_invalid(form)


class ProcessGpxDataView(generic.View):

    def get(self, request, object_id):
        instance = GpxModel.objects.get(pk=object_id)
        log.debug("Process GPX %s" % instance)

        content = instance.gpx
        gpxpy_instance = garmin2gpxpy(content)

        image, plt = generate_map(gpxpy_instance)

        temp = io.BytesIO()
        plt.savefig(temp, bbox_inches="tight")

        # Save gpx map file to model instance:
        instance.map_image.save("gpx", temp)

        messages.success(request, "GPX data saved to %s, ok." % instance)
        return HttpResponseRedirect("../")


class CalculateValuesView(generic.View):

    def get(self, request, object_id):
        instance = GpxModel.objects.get(pk=object_id)
        instance.calculate_values()
        instance.save()
        messages.success(request, "Values are calculated from GPX data")
        return HttpResponseRedirect("../")


@admin.register(GpxModel)
class GpxModelAdmin(admin.ModelAdmin):
    list_display = (
        # "image_tag",
        "svg_tag",
        "event",
        "short_start_address",
        "start_time",
        "human_length",
        "human_duration",
        "human_pace",
        "uphill",
        "downhill",
        "min_elevation",
        "max_elevation",
        "tracked_by"
    )
    list_display_links = (
        "event",
        "short_start_address",
    )
    readonly_fields = (
        "svg_tag_big",
        "image_tag",
        "svg_tag",
        "start_time",
        "start_latitude",
        "start_longitude",
        "finish_time",
        "finish_latitude",
        "finish_longitude",
        "start_coordinate_html",
    )

    fieldsets = (
        (_("Event"), {
            "fields": (
                "event",
                ("track_svg", "svg_tag_big"),
                ("map_image", "image_tag"),
            )
        }),
        (_("Start"), {
            "fields": (
                "start_time",
                "short_start_address",
                "full_start_address",
                    "start_coordinate_html",
                ("start_latitude", "start_longitude"),
            )
        }),
        (_("Finish"), {
            "fields": (
                "finish_time",
                "short_finish_address",
                "full_finish_address",
                ("finish_latitude", "finish_longitude"),
            )
        }),
        (_("GPX data"), {
            "classes": ("collapse",),
            "fields": (
                ("gpx", "points_no"),
            )
        }),
        (_("Values"), {
            "fields": (
                ("length", "duration", "pace"),
                ("uphill", "downhill"),
                ("min_elevation", "max_elevation"),
            )
        }),
    )
    # FIXME: Made this in CSS ;)
    formfield_overrides = {models.CharField: {'widget': forms.TextInput(attrs={'style': 'width:70%'})}}

    def get_urls(self):
        urls = super().get_urls()
        info = self.model._meta.app_label, self.model._meta.model_name
        urls = [
            url(r"^upload/$", self.admin_site.admin_view(UploadGpxFileView.as_view()), name="upload-gpx-file"),
            url(
                r"^(.+)/process/$",
                self.admin_site.admin_view(ProcessGpxDataView.as_view()),
                name="%s_%s_process-gpx-data" % info
            ),
            url(
                r"^(.+)/calculate_values/$",
                self.admin_site.admin_view(CalculateValuesView.as_view()),
                name="%s_%s_calculate-values" % info
            ),
        ] + urls
        return urls



# coding: utf-8



import pytest

# https://github.com/jedie/django-tools
from django_tools.unittest_utils.unittest_base import BaseTestCase
from django_tools.unittest_utils.user import TestUserMixin


@pytest.mark.django_db
class AdminAnonymousTests(BaseTestCase):
    """
    Anonymous will be redirected to the login page.
    """
    def test_login_en(self):
        response = self.client.get('/en/admin/', HTTP_ACCEPT_LANGUAGE='en')
        self.assertRedirects(response, expected_url='http://testserver/en/admin/login/?next=/en/admin/')

    def test_login_de(self):
        response = self.client.get('/de/admin/', HTTP_ACCEPT_LANGUAGE='de')
        self.assertRedirects(response, expected_url='http://testserver/de/admin/login/?next=/de/admin/')


@pytest.mark.django_db
class AdminLoggedinTests(TestUserMixin, AdminAnonymousTests):
    """
    Some basics test with the django admin
    """
    def test_staff_admin_index(self):
        self.login(usertype='staff')
        response = self.client.get('/en/admin/', HTTP_ACCEPT_LANGUAGE='en')
        self.assertResponse(response,
            must_contain=(
                'Django administration',
                'staff_test_user',
                'Site administration',
                'You don\'t have permission to edit anything.'
            ),
            must_not_contain=('error', 'traceback'),
            template_name='admin/index.html',
        )

    def test_superuser_admin_index(self):
        self.login(usertype='superuser')
        response = self.client.get('/en/admin/', HTTP_ACCEPT_LANGUAGE='en')
        self.assertResponse(response,
            must_contain=(
                'Django administration',
                'superuser',
                'Site administration',
                '/admin/auth/group/add/',
                '/admin/auth/user/add/',
            ),
            must_not_contain=('error', 'traceback'),
            template_name='admin/index.html',
        )



"""
    created 31.05.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
import io
from pathlib import Path

import gpxpy

from for_runners.gpx_tools.garmin2gpxpy import garmin2gpxpy





def parse_gpx_file(filepath):
    assert isinstance(filepath, Path)
    assert filepath.is_file()

    with filepath.open("r") as f:
        content = f.read()

    return parse_gpx(content)
